@@ -1,23 +1,23 @@
-#include <err.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
-#include "dedup.h"
+#include "misc.h"
 
 #define ROTL(x, y) (((x) << (y)) | ((x) >> (32 - (y))))
 
 struct chunker {
-	uint8_t *buf;
+	unsigned char *buf;
 	int fd;
-	size_t rpos;
-	size_t wpos;
-	size_t min_size;
-	size_t max_size;
+	size_t rp;
+	size_t wp;
+	size_t minsize;
+	size_t maxsize;
 	size_t mask;
-	size_t win_size;
+	size_t winsize;
+	uint32_t seed;
 };
 
 /*
@@ -29,7 +29,7 @@ struct chunker {
  * exactly 50% chance that a XOR operation would flip all the bits in
  * the hash.
  */
-static const uint32_t buz[] = {
+static uint32_t buztbl[] = {
 	0xbc9fa594,0x30a8f827,0xced627a7,0xdb46a745,0xcfa4a9e8,0x77cccb59,0xddb66276,0x3adc532f,
 	0xfe8b67d3,0x8155b59e,0x0c893666,0x1d757009,0x17394ee4,0x85d94c07,0xcacd52da,0x076c6f79,
 	0xead0a798,0x6c7ccb4a,0x2639a1b8,0x3aa5ae32,0x3e6218d2,0xb290d980,0xa5149521,0x4b426119,
@@ -66,135 +66,139 @@ static const uint32_t buz[] = {
 
 /* Buzhash: https://en.wikipedia.org/wiki/Rolling_hash#Cyclic_polynomial */
 static inline uint32_t
-buzh_init(uint8_t *buf, size_t size)
+hinit(unsigned char *buf, size_t size)
 {
 	uint32_t sum;
 	size_t i;
 
 	for (i = 1, sum = 0; i < size; i++, buf++)
-		sum ^= ROTL(buz[*buf], (size - i) % 32);
-
-	return sum ^ buz[*buf];
+		sum ^= ROTL(buztbl[*buf], (size - i) % 32);
+	return sum ^ buztbl[*buf];
 }
 
 static inline uint32_t
-buzh_update(uint32_t sum, uint8_t out, uint8_t in, size_t size)
+hupdate(uint32_t sum, unsigned char out, unsigned char in, size_t size)
 {
-	return ROTL(sum, 1) ^ ROTL(buz[out], size % 32) ^ buz[in];
-}
-
-static inline int
-match_pattern(struct chunker *chunker, size_t chunk_size, uint32_t sum)
-{
-	if (chunk_size >= chunker->max_size)
-		return 1;
-	if (chunk_size < chunker->min_size)
-		return 0;
-	return (sum & chunker->mask) == 0;
+	return ROTL(sum, 1) ^ ROTL(buztbl[out], size % 32) ^ buztbl[in];
 }
 
 static size_t
-get_chunk_size(struct chunker *chunker)
+cgetsize(struct chunker *c)
 {
-	size_t chunk_size, win_size, i;
+	size_t maxcsize, winsize, i;
 	uint32_t sum;
-	uint8_t *bp;
+	unsigned char *bp;
 
-	chunk_size = chunker->wpos - chunker->rpos;
-	win_size = chunker->win_size;
-	if (chunk_size < win_size)
-		return chunk_size;
+	maxcsize = c->wp - c->rp;
+	winsize = c->winsize;
+	if (maxcsize < winsize)
+		return maxcsize;
 
 	/*
 	 * To achieve better deduplication, we chunk blocks based on a
-	 * recurring pattern occuring on the data stream. A fixed window
-	 * of WINSIZE bytes is slid over the data, and a rolling hash is
-	 * computed for this window.
+	 * recurring pattern on the data stream. We slide a fixed window
+	 * of WINSIZE bytes over the data, and a rolling hash is computed
+	 * for this window.
 	 * When the rolling hash matches a given pattern the block is chunked
 	 * at the end of that window.
 	 */
-	bp = &chunker->buf[chunker->rpos];
-	sum = buzh_init(bp, win_size);
-	for (i = 0; i < chunk_size - win_size; i++) {
-		if (i > 0) {
-			uint8_t out = bp[i - 1];
-			uint8_t in = bp[i + win_size - 1];
+	bp = &c->buf[c->rp];
+	sum = hinit(bp, winsize);
+	for (i = 0; i < maxcsize - winsize; i++) {
+		size_t csize = i + winsize;
 
-			sum = buzh_update(sum, out, in, win_size);
+		if (i > 0) {
+			unsigned char out = bp[i - 1];
+			unsigned char in = bp[csize - 1];
+
+			sum = hupdate(sum, out, in, winsize);
 		}
 
-		if (match_pattern(chunker, i + win_size, sum))
-			return i + win_size;
+		if (csize < c->minsize)
+			continue;
+
+		if ((sum & c->mask) == 0)
+			return csize;
 	}
-	return chunk_size;
+	return maxcsize;
 }
 
 struct chunker *
-alloc_chunker(int fd, size_t min_size, size_t max_size,
-              size_t mask, size_t win_size)
+copen(int fd, size_t minsize, size_t maxsize,
+      size_t mask, size_t winsize, uint32_t seed)
 {
-	struct chunker *chunker;
+	struct chunker *c;
+	size_t i;
 
-	chunker = calloc(1, sizeof(*chunker));
-	if (chunker == NULL)
-		err(1, "calloc");
-
-	chunker->buf = calloc(1, max_size);
-	if (chunker->buf == NULL)
-		err(1, "calloc");
-
-	chunker->fd = fd;
-	chunker->min_size = min_size;
-	chunker->max_size = max_size;
-	chunker->mask = mask;
-	chunker->win_size = win_size;
-
-	return chunker;
-}
-
-void
-free_chunker(struct chunker *chunker)
-{
-	free(chunker->buf);
-	free(chunker);
-}
-
-ssize_t
-fill_chunker(struct chunker *chunker)
-{
-	uint8_t *bp;
-	ssize_t n;
-
-	bp = &chunker->buf[chunker->wpos];
-	n = xread(chunker->fd, bp, chunker->max_size - chunker->wpos);
-	chunker->wpos += n;
-	return chunker->wpos;
-}
-
-uint8_t *
-get_chunk(struct chunker *chunker, size_t *chunk_size)
-{
-	uint8_t *bp;
-
-	if (chunker->rpos == chunker->wpos) {
-		*chunk_size = 0;
+	c = calloc(1, sizeof(*c));
+	if (c == NULL) {
+		seterr("calloc: out of memory");
 		return NULL;
 	}
 
-	bp = &chunker->buf[chunker->rpos];
-	*chunk_size = get_chunk_size(chunker);
-	chunker->rpos += *chunk_size;
-	return bp;
+	c->buf = calloc(1, maxsize);
+	if (c->buf == NULL) {
+		free(c);
+		seterr("calloc: out of memory");
+		return NULL;
+	}
+
+	c->fd = fd;
+	c->minsize = minsize;
+	c->maxsize = maxsize;
+	c->mask = mask;
+	c->winsize = winsize;
+	c->seed = seed;
+
+	for (i = 0; i < sizeof(buztbl) / sizeof(buztbl[0]); i++)
+		buztbl[i] ^= c->seed;
+	return c;
 }
 
 void
-drain_chunker(struct chunker *chunker)
+cclose(struct chunker *c)
 {
-	uint8_t *src, *dst;
+	free(c->buf);
+	free(c);
+}
 
-	src = &chunker->buf[chunker->rpos];
-	dst = chunker->buf;
-	memmove(dst, src, chunker->wpos - chunker->rpos);
-	chunker->wpos -= chunker->rpos;
-	chunker->rpos = 0;
+ssize_t
+cfill(struct chunker *c)
+{
+	unsigned char *bp;
+	ssize_t n;
+
+	bp = &c->buf[c->wp];
+	n = xread(c->fd, bp, c->maxsize - c->wp);
+	c->wp += n;
+	return c->wp;
+}
+
+void *
+cget(struct chunker *c, size_t *csize)
+{
+	unsigned char *bp;
+
+	if (c->rp == c->wp) {
+		seterr("chunker underflow");
+		return NULL;
+	}
+
+	bp = &c->buf[c->rp];
+	*csize = cgetsize(c);
+	c->rp += *csize;
+	return bp;
+}
+
+size_t
+cdrain(struct chunker *c)
+{
+	unsigned char *src, *dst;
+
+	src = &c->buf[c->rp];
+	dst = c->buf;
+	memmove(dst, src, c->wp - c->rp);
+	c->wp -= c->rp;
+	c->rp = 0;
+	return c->wp;
 }
